/*
 * ICQSKMGR.C - pwICQ GUI Manager Plugin (Version with Skin support)
 */

 #define INCL_DOSPROCESS
 #define INCL_WIN

 #pragma strings(readonly)

 #include <time.h>
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
 #include <io.h>

 #include "icqskmgr.h"

/*---[ Prototipos ]--------------------------------------------------------------------------*/

 static int    _System executeSkin(HICQ icq,ULONG);
 static void   _System warning(HICQ, HUSER, USHORT, void *, const char *);

 static ULONG  _System loadSkin(HICQ, STATUSCALLBACK, int, char ** );
 static void   _System view(HICQ, ULONG, HMSG);
 static void   _System newMessage(HICQ, ULONG, USHORT, const char *, const char *);
 static void   _System ajustMsgDialog(HWND, HICQ, ULONG, USHORT, BOOL);
 static int    _System insertMenu(HICQ, USHORT, const char *, short, ULONG, MENUCALLBACK *);
 static int    _System openConfig(HICQ, ULONG, USHORT, PAGELOADER *);
 static BOOL   _System awayMessage(HICQ, ULONG, const char *, char *);
 static int    _System validateEditHelper(USHORT);
 static int    _System ddeRequest(HICQ, const char *, const char *, const char *);
 static int    _System popupMenu(HICQ, ULONG, USHORT);
 static int    _System openWindow(HICQ, HUSER, const char *, int (* _System)(HICQ, HWND, const char *, void *),void *);

/*---[ Constantes ]--------------------------------------------------------------------------*/

 static const SKINMGR caps = {          sizeof(SKINMGR),
                                        0,
                                        PROJECT,
                                        loadSkin,
                                        executeSkin,
                                        NULL,
                                        warning,
                                        view,
                                        newMessage,
                                        awayMessage,
                                        popupMenu,
                                        NULL,                   // popupUserList
                                        insertMenu,
                                        openConfig,
                                        validateEditHelper,
                                        NULL,                   // Event
                                        ddeRequest,
                                        openWindow              // int (* _System openWindow)(HICQ, HUSER, const char *, int (* _System)(HICQ, HWND, void *),void *);

                             };


 extern const SKINELEMENT       winElements[] =
 {
        0,             0,               0,                      NULL
 };

 extern const PMELEMENT       pmElements[] =
 {

        0,             0,               0,                      NULL
 };

/*---[ Statics ]-----------------------------------------------------------------------------*/

 static HWND    mainWindow      = NULLHANDLE;
 static HWND    mainFrame       = NULLHANDLE;
 extern HMODULE module          = NULLHANDLE;

HWND   g_UserMenu = NULLHANDLE;

/*---[ Implementacao ]-----------------------------------------------------------------------*/

 int _Export EXPENTRY icqpm_Configure(HICQ icq, HMODULE hmod)
 {
    char        key[20];
    char        *ptr;

printf("icqskmgr_Configure\n");

#ifdef DEBUG
    icqWriteSysLog(icq,PROJECT,"Loading pwICQ Skin manager " __DATE__ " " __TIME__ " (Debug Version)");
#else
    icqWriteSysLog(icq,PROJECT,"Loading pwICQ Skin manager Build " BUILD);
    printf("Loading pwICQ Skin manager Build " BUILD "\n");
#endif

#ifdef EXTENDED_LOG
    icqWriteSysLog(icq,PROJECT,"Extensive logging enabled");
#endif

    CHKPoint();

    if(icqQueryUserDataBlockLength() != sizeof(ICQUSER))
    {
       CHKPoint();
       icqWriteSysLog(icq,PROJECT,"Can't start protocol manager due to invalid pwICQ core version");
       return -1;
    }

    if(!icqRegisterSkinManager(icq,&caps))
    {
       CHKPoint();
       icqWriteSysLog(icq,PROJECT,"Can't register skin manager");
       return -2;
    }

    icqSetSkinDataBlock(icq,0);

    module = hmod;

#if 0
    key[19] = 0;
    strncpy(key,sknQueryIDString(),19);
    for(ptr = key;*ptr && *ptr != ':';ptr++);
    *ptr = 0;

    for(ptr = (char *) sknQueryIDString();*ptr && *ptr != ':';ptr++);

    if(*ptr)
       icqWriteSysLog(icq,key,ptr+1);
#endif

printf("icqskmgr_Configure done\n");

    return 0;
 }

 void _Export EXPENTRY icqpm_Timer(HICQ icq, void *lixo)
 {
    if(mainWindow)
       WinPostMsg(mainWindow,WM_USER+12,0,0);
 }

 void _Export EXPENTRY icqpm_SystemEvent(HICQ icq, void *lixo, USHORT event)
 {
    if (event!=35)
      printf( "icqskmgr_SystemEvent event %d\n", event);
    if(mainWindow)
       WinPostMsg(mainWindow,WM_USER+13,MPFROMSHORT(event),0);
 }

 void _Export EXPENTRY icqpm_UserEvent(HICQ icq, void *lixo, HUSER usr, USHORT event)
 {
printf( "icqskmgr_UserEvent event %d, usr %08x (%s)\n", event, usr, icqQueryUserNick(usr));
printf( "icqskmgr_UserEvent mode %d (%s)\n", usr->mode & 0xffff, icqQueryUserNick(usr));
    if(mainWindow)
       WinPostMsg(mainWindow,WM_USER+14,MPFROMSHORT(event),MPFROMLONG(usr->uin));
 }

 void _System view(HICQ icq, ULONG uin, HMSG m)
 {
    icqOpenMsgWindow(icq,uin,0,m,0,0);
 }

 char *querySkinName(HICQ icq, char *skin)
 {
    char        *ptr;
    int         sz;
    char        buffer[0x0100];

    icqLoadString(icq, "skin", "", skin, 0xFF);

    if(!*skin || access(skin,0))
    {
       icqQueryPath(icq,"skins\\",skin,0xFF);
       sz  = strlen(skin);
       ptr = skin + sz;
       sz  = 0xFF - sz;

       DBGMessage(skin);
       icqLoadString(icq, "skin", "default.skn", ptr, sz);

    }

    DBGMessage(skin);

    if(access(skin,0))
    {
       strcpy(buffer,"Can't find ");
       strncat(buffer,skin,0xFF);
       icqWriteSysLog(icq,PROJECT,buffer);
       return NULL;
    }
    strcpy(buffer,"Loading ");
    strncat(buffer,skin,0xFF);
    icqWriteSysLog(icq,PROJECT,buffer);

    DBGMessage(buffer);
    DBGMessage(skin);


    return skin;
 }

 ULONG _System loadSkin(HICQ icq, STATUSCALLBACK *startupStage, int numpar, char **param )
 {
    HAB         hab     = icqQueryAnchorBlock(icq);
    char        skin[0x0100];
    char*       szFont = "9.WarpSans";

    DBGMessage("Iniciando skin manager");
    icqWriteSysLog(icq,PROJECT,"Starting Skin Manager");
    printf("Starting Skin Manager\n");

    startupStage(icq,ICQSTARTUP_STARTINGSKIN);

    //if(sknInitialize(hab) && sknRegisterClass(hab,"pwICQ2Skin",sknProcedure,sizeof(void *)))
    if(WinRegisterClass(hab,"pwICQ2pm",sknProcedure,CS_SIZEREDRAW,sizeof(void *)))
    {
       ULONG frameFlags= FCF_DLGBORDER | FCF_SIZEBORDER | FCF_TITLEBAR | FCF_SYSMENU
                         | FCF_MINBUTTON  | FCF_CLOSEBUTTON | FCF_MENU
                         | FCF_TASKLIST | FCF_ICON | FCF_ACCELTABLE |FCF_NOBYTEALIGN;

       icqStartWindowTools(icq,hab);

       startupStage(icq,ICQSTARTUP_LOADINGSKIN);

       DBGTrace(icqQueryLogonStatus(icq));

       if(!icqQueryLogonStatus(icq))
       {
          startupStage(icq,-1000);
          icqLogonDialog(icq);
          if(!icqQueryLogonStatus(icq))
             return -2;
          startupStage(icq,-1001);
          startupStage(icq,ICQSTARTUP_LOADINGSKIN);
       }

       //mainWindow = sknCreateWindow("pwICQ2pm",querySkinName(icq, skin),&mainFrame,"pwICQ",module,
       //              FCF_DLGBORDER | FCF_SIZEBORDER | FCF_TITLEBAR | FCF_SYSMENU | FCF_MINBUTTON  | FCF_CLOSEBUTTON
       //              |FCF_TASKLIST|FCF_NOMOVEWITHOWNER|FCF_ICON|FCF_ACCELTABLE|FCF_NOBYTEALIGN,1);
       mainFrame = WinCreateStdWindow( HWND_DESKTOP,
                                WS_VISIBLE, &frameFlags,
                                (PSZ) "pwICQ2pm",
                                (PSZ) "pwicq2",
                                0,
                                module,
                                1,
                                &mainWindow
                            );
printf("mainWindow %08x\n", mainWindow);

   WinSetPresParam( mainWindow, PP_FONTNAMESIZE, strlen(szFont) + 1, szFont);

   g_UserMenu = WinLoadMenu( mainWindow, module, 1003);
/*
        WinSetWindowPos(mainWindow,
                HWND_TOP,
                100,
                100,
                200,
                400,
                SWP_MOVE | SWP_SIZE | SWP_SHOW);
*/
       DBGTracex(mainWindow);

       if(mainWindow)
       {
          icqRegisterConfigurationWindow(icq,hab);

          WinSendMsg(mainWindow,WM_USER+10,MPFROMP(icq),MPFROMLONG(mainFrame));
          startupStage(icq,ICQSTARTUP_SKINLOADED);
          if(icqInitialize(icq,startupStage))
             WinPostMsg(mainWindow,WM_CLOSE,0,0);
          WinPostMsg(mainWindow,WM_USER+19,0,0);
          return (ULONG) mainWindow;
       }
    }
    return 0;
 }

#ifdef DEBUG

 int _System testMenu(HICQ icq, ULONG uin, USHORT type,ULONG id)
 {
    DBGMessage("Menu callback");
    DBGTracex(type);
    DBGTrace(uin);

    return 0;
 }

#endif

 int _System executeSkin(HICQ icq, ULONG parm)
 {

    if(parm != mainWindow)
    {
       icqWriteSysLog(icq,PROJECT,"Unexpected parameter received");
       return -1;
    }

    icqRestoreWindow(mainFrame, icq, 0, "pos:MainWindow", 150, 375);

#ifdef DEBUG
    icqInsertMenuOption(icq, ICQMENU_SYSTEM, "Teste de menu", -1, 1, testMenu);
#endif

/*
    The icqExecuteWindowLoop() procedure basically does:

    while(WinGetMsg(hab,&qmsg,0,0,0))
       WinDispatchMsg(hab,&qmsg);

    I'm using it only to allow some extra debbuging in the message dialog box before the main
    message loop processing.

    pw 09/29/2001
*/
    icqExecuteWindowLoop(icq);

    icqStoreWindow(mainFrame, icq, 0, "pos:MainWindow");

    icqStopWindowTools(icq);
    icqTerminate(icq);

    DBGMessage("Skin terminado");

    icqWriteSysLog(icq,PROJECT,"Skin Manager stopped");
    DBGMessage("Skin manager terminado");

    return 0;
 }

 static void _System warning(HICQ icq, HUSER usr, USHORT id, void *x1, const char *x2)
 {
    ULONG uin   = 0;

    if(usr)
       uin = usr->uin;

    if(mainWindow)
       WinPostMsg(mainWindow,WM_USER+15,MPFROMLONG(uin),MPFROMSHORT(id));

 }

 void _System newMessage(HICQ icq, ULONG uin, USHORT type, const char *p1, const char *p2)
 {
#ifdef DEBUG

    DBGTracex(p1);
    if(p1)
       DBGMessage(p1);

    DBGTracex(p2);
    if(p2)
       DBGMessage(p2);

#endif

    icqOpenMsgWindow(icq, uin, type, NULL, p1, p2);

 }

 static int _System openConfig(HICQ icq, ULONG uin, USHORT type, PAGELOADER *loader)
 {
    DBGMessage("Abrir dialog de configuracao");
    return icqOpenDefaultConfigurationWindow(icq,uin,type,loader);
 }

 static int _System insertMenu(HICQ icq, USHORT menu, const char *text, short pos, ULONG icon, MENUCALLBACK *cbk)
 {
    MNUHANDLER mnu;
    MENUITEM mi;
    HWND hwndMenu, hwndPullDown;

    DBGMessage("Pediu insercao manual de menu");
    DBGTrace(menu);
    DBGTracex(WinWindowFromID(mainWindow,menu));

printf( "menu#%d, pos %d, '%s'\n", menu, pos, text);

   if (menu == 1003)
       hwndPullDown = g_UserMenu;
   else {
      hwndMenu = WinWindowFromID( mainFrame, FID_MENU);
      WinSendMsg(hwndMenu, MM_QUERYITEM, MPFROM2SHORT(menu, TRUE), (MPARAM) &mi);
      hwndPullDown = mi.hwndSubMenu;
   }

   mi.iPosition = MIT_FIRST;
   mi.afStyle = MIS_TEXT;
   mi.afAttribute = 0;
   mi.id = menu+pos;
   mi.hwndSubMenu = NULLHANDLE;
   mi.hItem = cbk;
   WinSendMsg( hwndPullDown, MM_INSERTITEM, (MPARAM) &mi, (MPARAM) text);


/*
    memset(&mnu,0,sizeof(MNUHANDLER));
    mnu.text    = text;
    mnu.after   = 9999;
    mnu.handler = (ULONG) cbk;
    mnu.itemID  = icon;
    sknInsertMenuHandler(mainWindow,menu,&mnu);
*/

    return 0;
 }

 static BOOL   _System awayMessage(HICQ icq, ULONG uin, const char *title, char *txt)
 {
    struct awayDialogParameters p;
    int                         f;

    CHKPoint();

    p.sz        = sizeof(struct awayDialogParameters);
    p.flag      = 0xFF;
    p.uin       = uin;
    p.title     = title;
    p.txt       = txt;
    p.icq       = icq;
    p.usr       = icqQueryUserHandle(icq,uin);

    WinPostMsg(mainWindow,WM_USER+18,MPFROMP(&p),0);

    for(f=0;p.flag && f<200;f++)
       DosSleep(10);

    if(p.flag)
    {
       icqSysLog(icq,"Timeout passing information to the main window");
       icqAbend(icq,0);
       return FALSE;
    }

    return TRUE;
 }

 static int _System validateEditHelper(USHORT sz)
 {
    if(sz != sizeof(MSGEDITHELPER))
       return 2;
    return icqValidateDefaultEditHelper(sz);
 }

 static int _System ddeRequest(HICQ icq, const char *application, const char *topic, const char *request)
 {
    icqWriteSysLog(icq,PROJECT,"Inactive DDE Requests");
    icqWriteSysLog(icq,PROJECT,request);
    return -1;
 }

 static int _System popupMenu(HICQ icq, ULONG uin, USHORT menu)
 {
    DBGMessage("Popup menu");

    if(!mainWindow)
       return -1;

    WinPostMsg(mainWindow,WM_USER+21,MPFROM2SHORT(menu,0),MPFROMLONG(uin));

    return 0;
 }

 static int _System openWindow(HICQ icq, HUSER usr, const char *id, int (* _System callback)(HICQ, HWND, const char *, void *),void *arg)
 {
    struct openWindowParameters parm = {        sizeof(struct openWindowParameters),
                                                TRUE,
                                                usr,
                                                id,
                                                callback,
                                                arg
                                        };
    int f;


    if(!mainWindow)
       return -1;

    DBGTracex(mainWindow);

    CHKPoint();
    WinPostMsg(mainWindow,WM_USER+22,(MPARAM) &parm,0);

    /* Espera a janela pegar o parametro */

    for(f=0;f<100 && parm.flag;f++)
       DosSleep(10);

    if(parm.flag)
    {
       icqWriteSysLog(icq,PROJECT,"Error passing message to the mainwindow");
       icqAbend(icq,0);
    }

    return 0;
 }


